package esmith::AD;

use strict;
use Net::LDAP;
use MIME::Base64();
use esmith::ConfigDB;
use esmith::AccountsDB;

no warnings ('qw');

=head1 NAME

esmith::AD - Interface to Samba 4 Active Directory for Koozali

=head1 SYNOPSIS

    use esmith::AD::User;
    use esmith::AD::Group;

    $ad_user = esmith::AD::User->new();
    @users = $ad_user->listUsers();

    $ad_group = esmith::AD::Group-new();
    @groups = $ad_group->listGroups();

    NOTE: Most methods in the class should not be run directly, but
          from a child class such as esmith::AD::User, or as noted 
          in the method specific documentation.

    
=head1 DESCRIPTION

This module provides an abstracted interface to Samba 4 Active Directory
for the Koozali SME Server.

Methods provided by this interface interacts with Samba 4 Active Directory
using the Net::LDAP perl module.  Understanding of Net::LDAP syntax is
not required to utilize the primary functionality provided by this interface,
but the full suite of Net::LDAP methods is provided by this module should 
the developer wish to use them.

=cut

sub new {
    my ($class, %params) = @_;
    my $self = {};

    bless ($self, $class);
    return ($self);
}

=head2 Fully Abstraced Methods

For all intents and purposes, this interface is the preferred approach to 
interact with the Koozali Active Directory. Included in this API are child 
classes for managing the different object types in the Active Directory 
including users, groups, and DNS entries.  Methods provided in this parent 
class should generally not be called directly (unless noted in the method 
specific documentation), but from one of these child classies.  This is because 
this API provides run-time binding of many of these parent methods to the 
object types they are presented with. See esmith::AD::User, esmith::AD::Group, 
and esmith::AD::DNS for more details.

Usage example:
       use esmith::AD::User;
       my $ad = esmith::AD->new;
       my $user_status = $ad->getUserStatus;


=head3 getADRecord {$name)

This method returns a hash reference of attribute / value pairs 
from the Active Directory for attributes supported by this method,
given an object name.  This method should not be called from this 
class, but one of the esmith::AD child classes.

Usage:  $hashref = $ad->getADRecord($name);

=cut

sub getADRecord {
    my ($self,$name) = @_;
    
    die $self->error("Must call this method on a esmith::AD child object.\n")
      if (ref($self) eq 'esmith::AD');
    die $self->error("Object name not specified.\n") unless $name;
    
    #Query record from AD
    my ($record) = $self->queryObjects('single',$name);
 
    #Verify that we found a record in the AD
    die $self->error("Unable to find \"$name\" in the Active Directory.\n") 
      unless ($record);

    #Build a hash of supported attributes and values 
    my @attributes = $self->attributes;
    my %adRecord = map {$_ => ($record->get_value($_))||''} @attributes;
   
    return (\%adRecord);
}

=head3 SID($name)

This method retrieves the SID for a specified Active Directory 
object name, decrypts it, and then returns it.  This method 
must be called from a esmith::AD child and not directly
from this class.

Usage: my $sid = $ad->getSID('username');

=cut

sub getSID {
    my ($self,$name) = @_;
    
    #Don't allow method from this class
    die $self->error("Must call this method on a esmith::AD child object.\n")
      if (ref($self) eq 'esmith::AD');
    die $self->error("Object name not specified.\n") unless $name;

    my ($record) = $self->queryObjects('single',$name);

    die $self->error("Unable to find \"$name\" in the Active Directory.\n")
      unless ($record);
   
    my $encryptedSID = $record->get_value('objectSid');

    return($self->sidToString($encryptedSID));
}

=head3 getStatus($name)

This method returns the status, enabled or disabled, for various entries
in the Active Directory. This method must be called from a esmith::AD 
child and not directly from this class.

Usage:  my $status=$ad->getStatus('username');

=cut

sub getStatus {
    my ($self,$name) = @_;
    
    #Don't allow this method from this class
    die $self->error("Must call this method on a esmith::AD child object.\n")
      if (ref($self) eq 'esmith::AD');
    die $self->error("Object name not specified.\n") unless $name;
    
    #Lookup entry in the active directory
    my ($record) = $self->queryObjects('single',$name);
    
    die $self->error("Unable to find \"$name\" in the Active Directory.\n") 
      unless ($record);
    
    #Determine if Active Directory entry is enabled or disabled
    my $userAccountControl = $record->get_value('userAccountControl');
   
    if ($userAccountControl ==  512) {return('enabled');}
    else {return ('disabled');}
}

=head3 listObjects($set)

This method returns a list of object names (cn) from the Active Directory.  
The method is meant to be called from an esmith::AD child and not directly 
from the parent class, esmith::AD.  This method also queriesi subsets of 
objects:

   - all:  All objects within the search objectClass;
   - builtin: Objects defined during the AD provisioning;
   - koozali: Objects defined by this API.

Usage: use esmith::AD::User;
 
       my $ad = esmith::AD;User->new();
       my @objects = $ad->listObjects('koozali');

=cut

sub listObjects {
    my ($self,$set) = @_;

    ## Determine what the subset of objects to search for 
    unless (grep {$set eq $_} qw(all builtin koozali)) {
        $set = 'all';
    }
    die $self->error("Must call this method on a esmith::AD child object.\n")
      if (ref($self) eq 'esmith::AD');

    my @records = $self->queryObjects($set); 
 
    die $self->error("Unable to query objects from Active Directory.\n")
      unless (@records);
 
    my @objects = map { $_->get_value('cn') } @records;

    return(@objects);
}

=head3 setAttr($type,$name,$hashRef)

This method sets or modifies attribute /  value pairs for an Active Directory
object given object type, object name, and a hash reference to the values 
to set or modify. It should be noted that this method will only update 
attribute / value pairs as defined the validData method.  

NOTE:  This method dies is data fails validation check.  This may need
          to be changed to provide better feedback to server-panels and 
          what not.

FIXME: 1)NEED TO FIX THE CODE THAT VALIDATES DAT APRIOR TO UPDATING
       2)The name attribute is actually givenName + sn (surname) attribute
         with a space between.  We should consider forcing this instead of 
         letting it be changed free form as we are now.
=cut 

sub setAttr {
    my ($self,$name,$attrRef) = @_;
   
    #Don't allow this method from this class
    die $self->error("Must call this method on a esmith::AD child object.\n")
      if (ref($self) eq 'esmith::AD');
    die $self->error("Entity to modify not specified.\n") unless ($name);
    die $self->error("No attributes set to modify for $name\n") 
      unless (ref($attrRef) eq 'HASH');
    
    
    ##Validate attributes and values to update
    foreach my $key (keys %$attrRef) {
        die $self->error("Unsupported attribute \"$key\".\n") 
          unless ($self->validData->{$key});
        
        ###FIX ME:  Add regex check for actual values
    }    

    #Query Record to update from AD
    my ($record) = $self->queryObject($name);
    die $self->error("Unable to query \"$name\" from Active Directory\n.")
      unless ($record);   
 
    my $result = $self->updateAD($record,$attrRef);
    die $self->error("Unable to update \"$name\" record\n") unless ($result);

    return (1);
}

=head3 setPosix($type,$name)

This method designates an Active Directory object as a Posix object given
object type and name.  Currently, this method support the types user and 
group.  

=cut 

sub setPosix {
    my ($self,$name) = @_;

    #Don't allow this method from this class
    die $self->error("Must call this method on a esmith::AD child object.\n")
      if (ref($self) eq 'esmith::AD');
 
    #Pull record from AD
    my ($record) = $self->queryObject($name);
    die $self->error("Unable to find \"$name\" in the Active Directory.\n") 
      unless ($record);
    
    #Set the Posix objectClass
    my $result = $self->updateAD($record,
                                 {'objectClass'=>$self->{'objectClass'}});
    die $self->error("Unable to update \"$name\" record.\n") unless ($result);

    return(1);
}


=head2 Partially Abstraced Methods

Methods providing a thin abstracted connection to Koozali Active Directory
using Net::LDAP.  These methods are designed to connect directly to the 
Koozali Active Directory and return Net::LDAP objects.  The developer should
generally not need to call these Partiall Abstracted Methods directly, as 
this class includes Fully Abstracted Methods that provide Active Directory 
functionality needed to manage the Koozali SME Server.  However, the 
developer may utilize these Partially Abstracted Methods for more fine 
grained interacting with the Active Directory using Net::LDAP.

=head3 connect()

This method returns a connection object to the Active Directory
using configuration parameters stored in the Koozali configuration
database and the currently provisioned Active Directory domain.

Domain administrator credentials needed to access the Active Directory
are taken from a password encrypted to the local filesystem in 
/etc/samba/AD.pw.  A method to decrypt this stashed password is 
provided by the esmith::AD::getADPass().

    Usage:
           $self = esmith::AD->new;
           $ad_connection_object = $self->connect;

=cut

sub connect {
    my $self = shift;
    
    #Pull Samba Domain and AD IP config db 
    my $domain = $self->domain;
    my $sme_ad_host=$self->hostIP;

    #Pull AD password from local storage
    my $ad_password = $self->getADPass;
    
    #Open a connection to the Active Director and return the connection
    my $ad = Net::LDAP->new($sme_ad_host);

    #Bind to the AD and return the connection object
    my $bind = $ad->bind($domain . "\\ad_admin", password=>$ad_password);
    if($bind->code) {
        die $self->error($bind->server_error);
    }
    
    return ($ad); 
}
 
=head3 runQuery($filter)

This method runs a query of the Active Directory given a query filter
formated as specified by Net::LDAP and returns a list of Net::LDAP::Entry
objects. It is not necessary to bind to the Active Directory before calling 
this meathod, as this method calls the connect() method itself.

Usage:
       use esmith::AD;
 
       my $ad = esmith::AD->new;
       my @records = $ad->runQuery($filter);

=cut 

sub runQuery {
    my ($self,$filter) = @_;

    die $self->error('Query filter no specified.') unless $filter;

    ##Connect to AD 
    my $ad = $self->connect;
  
    ##Send query to the Active Directory
    my $query=$ad->search(
                          base   => 'DC=domain,DC=com',
                          filter => "$filter"
                         );
    $ad->unbind;
    if($query->code) { die $self->error($query->server_error);}

    return ($query->entries);
}

=head3 updateAD($record,$attRef)

This method updates an Active Directory record, given a Net::LDAP::Entry
object and a hash reference of data to update.  This method does not check 
the contents of the record to update nor the hash reference prior to trying 
to upate the Active Directory other than schema checks done by the LDAP 
server. The method will return 1 if the update was successful and will die 
with a LDAP server error is there was a problem.

Usage:    my $update_result = $self->updateAD($record);

=cut 

sub updateAD {
    my ($self,$record,$attrRef) = @_;

    die $self->error("Record object to update not specified.\n")
      unless ($record =~ /Net::LDAP::Entry/); 
    die $self->error("Update hash reference no specified.\n")
      unless (ref($attrRef) eq "HASH"); 

    #Bind to AD and update 
    my $ad = $self->connect();

    $record->replace(%$attrRef); #Update local record
    my $result = $record->update($ad); #update LDAP
    
    if($result->code()) { die $self->error($result->server_error); }
    $ad->unbind;
    
    return(1);
}

##-----------------------------------------------------------
##HELPER METHODS 
##
##The methods in the following section are not documented 
##other than the note comments contained in this code, as they 
##are meant to support other methods contained in this class.
##The developer may use these methods, but should examine the
##code closely to achieved desired results.
##-----------------------------------------------------------

#------------------------------------------------------------
#Method: buildFilter($set,$value)
#  This method builds a query filter by run-time binding
#  to esmith::AD child query elements
#-----------------------------------------------------------
sub buildQuery {
    my ($self,$class,$set,$value)=@_;

    #Set dynamic LDAP query elements
    $self->{'query'}->{'single'} =  "&(cn=$value)(objectClass=group)";
    $self->{'query'}->{'member'} =  "&(objectClass=group)(memberOf=$value)";

    $set = 'all' unless (grep {$set eq $_} qw(all builtin koozali));

    my $filter = $self->{'query'}->{$class} . $self->{'query'}->{$set};
    $filter   .= $self->{'query'}->{'noMachine'};
    
    return($filter)
}

#------------------------------------------------------------
#Method: error()
#-----------------------------------------------------------
sub error {
    my ($self,$error)=@_;
    
    return ((caller(1))[3] . " error: " . $error);
}

#------------------------------------------------------------
#Method:  domain()
#   This method returns the Samba domain from the smb 
#   configuration dbase.  This method is not meant to be
#   used external to this class.
#------------------------------------------------------------
sub domain {
    my $self = shift;
    
    my $cdb = esmith::ConfigDB->open_ro;
    my $domain = $cdb->get_prop('smb','Workgroup') ||
      die $self->error("Unable to determine AD domain name from ConfigDB\n");
    $cdb->close;
    
    return($domain);
}

#------------------------------------------------------------
#Method:  getADPass()
#   This method returns the locally stashed and encrypted
#   AD password.
#-----------------------------------------------------------
sub getADPass {
    my $self = shift;

    open( ADPW,"</etc/samba/AD.pw") ||
      die $self->error("Unable to read AD stashed password " .
                       "from local storage.\n");
    my $encrypted = <ADPW> || '';
    chomp $encrypted;
    close ADPW;
    return (MIME::Base64::decode($encrypted));
}

#------------------------------------------------------------
#Method:  hostIP()
#   This method returns the Samba domain from the smb
#   configuration dbase. This method is not meant to be
#   used external to this class.
#-----------------------------------------------------------
sub hostIP {
    my $self=shift;
    
    my $cdb = esmith::ConfigDB->open_ro;
    my $hostip = $cdb->get('LocalIP')->value ||
      die $self->error("Unable to determine AD server IP from ConfigDB.\n");
    $cdb->close;
    
    return($hostip);
}

#------------------------------------------------------------
#Method:  sidToString
#
#   This method decryptes the Samba SIDs and returns a
#   clear text version of the SID
#-----------------------------------------------------------
sub sidToString {
    my ($self, $sid) = @_;

    return undef
        unless unpack("C", substr($sid, 0, 1)) == 1;

    ##FIX ME:  This is a check for the SID length, but it is FUBAR.
    ##Fortunately this function seems to be working fine without this
    ##check, but it would be nice to figure out why this is failing
    #return undef
    #    unless length($sid) == 8 + 4 * unpack("C", substr($sid, 1, 1));
    
    my $sid_str = "S-1-";
    
    $sid_str .= (unpack("C", substr($sid, 7, 1)) +
                (unpack("C", substr($sid, 6, 1)) << 8) +
                (unpack("C", substr($sid, 5, 1)) << 16) +
                (unpack("C", substr($sid, 4, 1)) << 24));
    
    for my $loop (0 .. unpack("C", substr($sid, 1, 1)) - 1) {
        $sid_str .= "-" . unpack("I", substr($sid, 4 * $loop + 8, 4));
    }

    return $sid_str;
}

=head1 AUTHOR

Greg Zartman, Koozali Foundation <gzartman@koozali.org>

=cut

=head1 SEE ALSO

Net::LDAP

=cut

1;
