package esmith::AD::Group;

use strict;
use Net::LDAP;
no warnings ('qw');


=head1 NAME

esmith::AD::Group -- Sub-Class to the Koozali esmith::AD API

=head1 SYNOPSIS

    use esmith::AD
    
    my $ad = esmith::AD->new();
    my @groups = $ad->groups;

    
=head1 DESCRIPTION

This module provides esmith::AD methods related to group management in 
the Koozali Active Directory.  Like the parent class, esmith::AD, this
module uses Net::LDAP to communicate with the Active Directory.  Methods
provided in this module abstract Net::LDAP so that knowledge of this
module is not required.

=cut

sub new {
    my ($class, %params) = @_;
    my $queryElements = {
                         'all'       => '',
                         'builtin'   => "(!(objectClass=posixAccount))",
                         'koozali'   => "(objectClass=posixAccount)",
                         'member'    => '',
                         'noMachine' => "(!(objectClass=computer))",
                         'object'    => "&(objectClass=group)",
                         'single'    => ''
                        };
    my $objectClass = ['top','group','posixAccount'];
    my $self = {
                'query'       => $queryElements,
                'objectClass' => $objectClass
               };

    bless ($self, $class);
    return ($self);
}

=head2 Methods

=head3 attributes()

This method returns a list of Active Directory group attributes managed
by this API.

Usage:  @ad_group_attributes = $ad->attributes();

=cut

sub attributes {
    my $self = shift;
    
    my %dataHash = $self->validData;

    return (keys %dataHash);
}

=head3 createGID($groupname)

This method calculates a unix GID for an Active Directory group
based upon the SID for the Active Directory object.

Usage:  my $gid = $ad->createGID($groupname);

=cut

sub createGID {
    my ($self,$group) = @_;

    die $self->error("Groupname not specified.\n") unless $group;     
    
    #Pull SID from the AD   
    return (0) unless (my $SID = $self->getSID($group));
   
    #Return GID = RID + 2000    
    my $RID = (split(/-/,$SID))[7];
    return($RID+2000);
}   

=head3 doesGroupExist($groupname)

This method checks the Active Directory for the existance of a group name 
passed as a parameter to the method.  The method returns 1 if the 
group name exists in the Active Directory and 0 if it does not.

Usage:  $group_check = $ad->doesGroupExist('group_name');

=cut

sub doesGroupExist {
    my ($self,$group) = @_;
   
    my @groups = $self->listGroups('all'); 

    if(grep { $group eq $_ } @groups) { return (1); }
    else{ return(0); }
}

=head3 getGID($groupname)

This method returns the unix GID from the Active Directory, 
given a groupname.

Usage:  my $gid = $ad->getGID('group_name');

=cut

sub getGID { ##TO DO
    my ($self,$name) = @_;
    
    die $self->error("Groupname not specified.\n") unless $name;     
       
###FIX ME:  We are now pulling the GID from the AD
    
    return();
}   

=head3 getGroupAttr($group,$attr)

This method returns the value of the specified attribute for a group
listed in the Active Directory.

Usage:  $value = $ad->getGroupAttr('group','attribute');

=cut

sub getGroupAttr {

    my($self,$group,$attr)=@_;

    #Check that the search attribute is supported
    die $self->error("Attribute \"$attr\" not supported\n")
      unless ($self->validData->{$attr});
   
    #Get attribute value from the Active Directory
    my $value = $self->getADRecord($group)->{$attr} || '';
    
    return ($value);
}

=head3 getMembers($group)

###FIX ME:  Rewrite
This method returns a list group membersthe value of the specified attribute for a group
listed in the Active Directory.

Usage:  $value = $ad->getGroupAttr('group','attribute');

=cut

sub getMembers { ##TO DO -- May want to change to just "getMembers"

    my($self,$group,$attr)=@_;

    #Check that the search attribute is supported
    die $self->error("Attribute \"$attr\" not supported\n")
      unless (grep {$attr eq $_} ($self->attributes('group')));
   
    #Get attribute value from the Active Directory
    my $value = ($self->getADRecord('group',$group))->{$attr} || '';
    
    return ($value);
}

=head3 listGroups($type)

This method returns a list of groups based upon the type of group 
specified as a parameter to the method.  Valid types include:
 - builtin:  Groups defined during the Active Directory 
             provisioning (e.g., Administrator);
 - koozali:  Groups defined in the Koozali API.
 - all:  All groups.

If the method doesn't recieve a valid type, then it will return a list
of all groups in the Active Directory

=cut

sub listGroups {
    my ($self, $type) = @_;

    return($self->listObjects($type)); 
}

=head3 queryMembers($set,$value)

###FIX ME:  Need to rewire
This method is an alias method to the esmith::AD::listObjects method.
It returns a list of usernames from the Active Directory, and also
subsets of users:

 - builtin:  Users defined during the Active Directory
             provisioning (e.g., Administrator);
 - koozali:  Users defined in the Koozali API.
 - all:  All users (default).

Usage:
       use esmith::AD::User;
       my $ad = esmith::AD::User->new();
       my @users = $ad->listUsers('koozali');

=cut

sub queryMembers {
    my ($self,$set,$value)=@_;
    
    my $queryFilter = $self->buildQuery('member',$set,$value) || '';    
 
    return ($self->runQuery($queryFilter));
}


=head3 queryObjects($set,$value)

###FIX ME:  Need to rewire
This method is an alias method to the esmith::AD::listObjects method.
It returns a list of usernames from the Active Directory, and also
subsets of users:

 - builtin:  Users defined during the Active Directory
             provisioning (e.g., Administrator);
 - koozali:  Users defined in the Koozali API.
 - all:  All users (default).

Usage:
       use esmith::AD::User;
       my $ad = esmith::AD::User->new();
       my @users = $ad->listUsers('koozali');

=cut

sub queryObjects {
    my ($self,$set,$value)=@_;
    
    my $class = 'object';
    if (defined $value) {$class = 'single'};
    
    my $queryFilter = $self->buildQuery($class,$set,$value) || '';    
 
    return ($self->runQuery($queryFilter));
}

=head3 setGroupAttr($group,$attr,$value)

This method sets or modifies an attribute value for the specified 
groupname.

Usage:  $ad->setGroupAttr('groupname','attribute','value')

=cut 

sub setGroupAttr {
    my ($self,$group,$attr,$value) = @_;
   
    if ($self->setAttr($group,{$attr=>$value})) { return(1); }
    else { 
        die->error("Unable to modify \"$attr\" for groupname \"$group\".\n");
    }
}

=head3 setGroupMembers($group,\@members)

###FIX ME:  NEED TO REWITE
This method sets or modifies an attribute value for the specified 
groupname.

Usage:  $ad->setGroupAttr('groupname','attribute','value')

=cut 

sub setGroupMembers {   ##TO DO
    my ($self,$group,$attr,$value) = @_;
   
    if ($self->setAttr($group,{$attr=>$value})) { return(1); }
    else { 
        die->error("Unable to modify \"$attr\" for groupname \"$group\".\n");
    }
}



=head3 setPostixGroup ($group)

This method designates a group as a Posix Group.  The Posix group type
represents the groups managed by Koozali and are those groups that are 
displayed in the server-manager

Usage:  $ad->setPosixGroup('groupname')

=cut

sub setPosixGroup { 
    my ($self,$name) = @_;

    return($self->setPosix($name));
}

=head3 validData()

This method returns a hash reference of valid User attributes and value formats.
Value formats are qr// quoted regex stored in the hash values.

Usage:  $hashRef = $ad->validData()

=cut

sub validData {
    my $self = shift;

    return ({});
}

=head1 COPYRIGHT

Copyright (c) 2014 Koozali Foundation, Inc.
This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 AUTHOR

Greg Zartman, Koozali Foundation <gzartman@koozali.org>

=head1 SEE ALSO

Net::LDAP, esmith::AD

=cut

1;
