package esmith::AD::User;

use strict;
use base 'esmith::AD';
no warnings ('qw');

=head1 NAME

esmith::AD::User - Sub-Class to the Koozali esmith::AD API

=head1 SYNOPSIS

    use esmith::AD::User;
   
    my $ad = esmith::AD::User->new();
    my @users = $ad->listUsers;
 
=head1 DESCRIPTION

This module provide esmith::AD::User methods for management of users stored
in the Koozali Active Directory.  These methods are the preferred methods to 
interact with user objects in the Active Directory.  Many methods in this 
class inherit functionality from the esmith::AD parent class and provide 
run-time binding of user objects to these parent methods. Methods provided 
in this module abstract Net::LDAP so that knowledge of this module is not 
required.

=cut

sub new {
    my ($class, %params) = @_;
    my $queryElements = {
                         'all'       => '',
                         'builtin'   => "(!(objectClass=posixAccount))",
                         'koozali'   => "(objectClass=posixAccount)",
                         'member'    => '',
                         'noMachine' => "(!(objectClass=computer))",
                         'object'    => "&(objectClass=user)",
                         'single'    => ''
                        };
    my $objectClass = ['top','person','organizationalPerson','user',
                       'posixAccount']; 
    my $self = {
                'query'       => $queryElements,
                'objectClass' => $objectClass
               };

    bless ($self, $class);
    return ($self);
}

=head2 Methods

=head3 attributes()

This method returns a list of Active Directory user attributes managed
by this API.

Usage:  @ad_user_attributes = $ad->attributes();

=cut

sub attributes {
    my $self = shift;

    my %dataHash=$self->validData;

    return (keys %dataHash);
}

    
=head3 createUID($name)

This method calculates a unix UID for an Active Directory user 
based upon the SID for the Active Directory object.

Usage:  my $uid = $ad->createUID($username);

=cut

sub createUID {
    my ($self,$name) = @_;
    
    die $self->error("Username not specified in createUID method.\n") 
      unless $name;     
    
    #Pull SID from the AD   
    return (0) unless (my $SID = $self->getSID('user',$name)); 
   
    #Return UID = RID + 2000    
    my $RID = (split(/-/,$SID))[7];
    return($RID+2000);
}

=head3 doesUserExist($username)

This method checks the Active Directory for the existance of a username passed
as a parameter to the method.  The method returns 1 if the username exists in 
the Active Directory and 0 if it does not.

Usage:  $user_check = $ad->doesUserExist('group_name');

=cut

sub doesUserExist {
    my ($self,$user) = @_;

    my @users = $self->listUsers('all');
    
    if (grep { $user eq $_ } @users) { return(1); }
    else{return(0);}
}

=head3 getUID($username)

This method returns the unix UID for a user from the Active Directory.

Usage: my $uid = $ad->UID('username'); 

=cut

sub getUID {
    my ($self,$user)=@_;

###FIX ME:  Pull UID from AD instead of calculating it.
    
    return();
}

=head3 getUserAttr($username,$attr)

This method returns the value of the specified attribute for a user
listed in the Active Directory.

Usage:  $value = $ad->getUserAttr('uername','attribute');

=cut

sub getUserAttr {

    my($self,$username,$attr)=@_;

    ##Check that the search attribute is supported
    die $self->error("Attribute \"$attr\" not supported.\n")
      unless (grep {$attr eq $_} ($self->attributes('user')));
   
    #Get attribute value from the Active Directory
    my $value = ($self->getADRecord($username))->{$attr} || '';
    return ($value);
}

=head3 getUserGroups($username)

This method returns a list of the Active Directory groups that a user
is a member of.  

Usage:  @ad_groups=$ad->getUserGroups('username');

=cut

sub getUserGroups {  ##TO DO
    my ($self,$user) = @_;

    #Lookup entry in the active directory
    my ($record) = $self->query('user',$user);
    
    die $self->error("Unable to find \"$user\" in the Active Directory.\n") 
      unless ($record);
    
    #Get memberOf values from AD record object
    my @results = $record->get_value('memberOf');
   
    return (map {(split(/[=,]/,$_))[1]} @results);
}

=head3 getUserStatus($username)

This method returns user account status, enabled or disabled, for a 
specified username.

Usage:  $user_account_status=$ad->getUserStatus('username');

=cut

sub getUserStatus {
    my ($self,$user)=@_;

    return ($self->getStatus($user));
}

=head3 listUsers($set)

This method is an alias method to the esmith::AD::listObjects method.
It returns a list of usernames from the Active Directory, and also
subsets of users:  
 
 - builtin:  Users defined during the Active Directory 
             provisioning (e.g., Administrator);
 - koozali:  Users defined in the Koozali API.
 - all:  All users (default).

Usage: 
       use esmith::AD::User;
   
       my $ad = esmith::AD::User->new();
       my @users = $ad->listUsers('koozali');

=cut

sub listUsers {
    my ($self,$set) = @_;
    
    return($self->listObjects($set)); 
}

=head3 queryMembers($set,$value)

###FIX ME:  Need to rewire
This method is an alias method to the esmith::AD::listObjects method.
It returns a list of usernames from the Active Directory, and also
subsets of users:

=cut

sub queryMembers {
    my ($self,$set,$value)=@_;

    my $queryFilter = $self->buildQuery('member',$set,$value) || '';

    return ($self->runQuery($queryFilter));
}

=head3 queryObjects($set,$value)

###FIX ME:  Need to rewire
This method is an alias method to the esmith::AD::listObjects method.
It returns a list of usernames from the Active Directory, and also
subsets of users:  
 
 - builtin:  Users defined during the Active Directory 
             provisioning (e.g., Administrator);
 - koozali:  Users defined in the Koozali API.
 - all:  All users (default).

Usage: 
       use esmith::AD::User;
  
       my $ad = esmith::AD::User->new();
       my @users = $ad->listUsers('koozali');

=cut

sub queryObjects {
    my ($self,$set,$value)=@_;

    my $class = 'object';
    if (defined $value) {$class = 'single'};

    my $queryFilter = $self->buildQuery($class,$set,$value) || '';

    return ($self->runQuery($queryFilter));
}

=head3 setPostixUser ($user)

This method designates a user as a Posix User.  The Posix user type
represents the users managed by Koozali and are those groups that are 
displayed in the server-manager

Usage:  $ad->setPosixUser('username')

=cut 

sub setPosixUser {
    my ($self,$name) = @_;

    return($self->setPosix($name));
}


=head3 setUserAttr($user,$attr,$value)

This method sets or modifies an attribute value for the specified 
username.

Usage:  $ad->setUserAttr('username','attribute','value')

=cut 

sub setUserAttr {
    my ($self,$user,$attr,$value) = @_;
    
    if ($self->setAttr($user,{$attr=>$value})) { return(1); }
    else { die->error("Unable to modify \"$attr\" for user \"$user\".\n");}
}

=head3 setUserGroups($username,$groupRef)

This mthod sets the groups that a user is a member of give the username
and an array reference to the group names a user is a member of.

=cut 

sub setUserGroups {###TO DO
    my ($self,$name,$groupRef) = @_;
   
    #Check input parameters 
    die $self->error("Username not specified.\n") unless ($name);
    die $self->error("List of user groups not specified for $name\n") 
      unless (ref($groupRef) eq 'ARRAY');
    
    my @groups = @$groupRef;
  
    #Verify that groups exist in the AD

    #Call connect as we'll need this Net::LDAP object later
    my $ad = $self->connect();
     
    #Query AD using Net::LDAP syntax, as we called connect
    my $query=$ad->search(
                          base   => 'DC=domain,DC=com',
                          filter => "&(cn=$name)(objectClass=user)"
                        );
    if($query->code()) {
        die $self->error("Active Directory query error: ") .
        ($query->server_error);
    }
    die $self->error("Unable to query \"$name\" from the Active Directory.\n")
      unless (my $entry = $query->entry());

    #Modify Active Directory record and return true if successful, else die
    #with error message from the Active Directory if update failed    
    
    #$entry->replace(%dataHash);
    my $result = $entry->update($ad);

    if($result->code()) {
       die $self->error("Could not update \"$name\": ") . 
         ($result->server_error);
    }
    return (1);
}

=head3 validData()

This method returns a hash reference of valid User attributes and value formats.
Value formats are qr// quoted regex stored in the hash values.  

Usage:  $hashRef = $ad->validData()

=cut 

sub validData {
    my $self = shift;

    return ({
               'company'                    => 1,
               'countryCode'                => 1,
               'department'                 => 1,
               'description'                => 1,
               'displayName'                => 1,
               'gidNumber'                  => 1,
               'givenName'                  => 1,
               'homeDirectory'              => 1,
               'initials'                   => 1,
               'loginShell'                 => 1,
               'mail'                       => 1,
               'name'                       => 1,
               'physicalDeliveryOfficeName' => 1,
               'sn'                         => 1,
               'telephoneNumber'            => 1,
               'title'                      => 1,
               'uidNumber'                  => 1,
               'unixHomeDirectory'          => 1,
               'wWWHomePage'                => 1
            });
}


=head1 COPYRIGHT

Copyright (c) 2014 Koozali Foundation, Inc.
This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 AUTHOR

Greg Zartman, Koozali Foundation <gzartman@koozali.org>

=head1 SEE ALSO

Net::LDAP

=cut

1;
